/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "recordbuffer.h"

namespace bit
  {

  RecordBuffer::RecordBuffer(size_t initial_size, Growth growth, size_t sizemax):
      Buffer(initial_size, growth, sizemax),
      m_signal_field_changed_emitting(false)
  {}

  RecordBuffer::RecordBuffer(RecordBase::pointer record, size_t initial_size, Growth growth, size_t sizemax):
      Buffer(initial_size, growth, sizemax),
      m_record(record),
      m_signal_field_changed_emitting(false)
  {}

  RecordBuffer::RecordBuffer(void* external_data, size_t data_octets, DataMode datamode, Growth growth, size_t sizemax):
      Buffer(external_data, data_octets, datamode, growth, sizemax),
      m_signal_field_changed_emitting(false)
  {}

  RecordBuffer::RecordBuffer(RecordBase::pointer record, void* external_data, size_t data_octets, DataMode datamode, Growth growth, size_t sizemax):
      Buffer(external_data, data_octets, datamode, growth, sizemax),
      m_record(record),
      m_signal_field_changed_emitting(false)
  {}

  RecordBuffer::~RecordBuffer()
  {}

  FieldBuffer RecordBuffer::operator[](size_t field_id) throw (error::no_record)
  {
    if (m_record)
      return FieldBuffer(*this, m_record->field(field_id));
    else
      throw error::no_record();
  }

  FieldBuffer RecordBuffer::operator[](std::string field_id) throw (error::no_record)
  {
    if (m_record)
      return FieldBuffer(*this, m_record->field(field_id));
    else
      throw error::no_record();
  }

  FieldBuffer RecordBuffer::operator[](FieldBase::iterator iter)
  {
    return FieldBuffer( *this, iter.pointer() );
  }

  FieldBuffer RecordBuffer::operator[](FieldBase::pointer pointer)
  {
    return FieldBuffer( *this, pointer );
  }

  void RecordBuffer::set_record(RecordBase::pointer record ) throw ()
  {
    m_record = record;
    this->on_record_changed();
  }

  void RecordBuffer::clear_record() throw ()
  {
    m_record.reset();
    this->on_record_changed();
  }

  RecordBase::pointer RecordBuffer::record( )
  {
    return m_record;
  }

  sigc::signal< void, FieldBuffer > RecordBuffer::signal_field_changed( )
  {
    return m_signal_field_changed;
  }

  RecordBuffer& RecordBuffer::operator =( const RecordBuffer & other )
  {
    m_record = other.m_record;
    Buffer::operator=(other);
    return *this;
  }

  void RecordBuffer::on_field_changed( FieldBase::pointer field )
  {
    FieldBase::pointer current = field;

    if (m_signal_field_changed_emitting)
      {
        //we only need to mark each field for emitting once, so check to see if we already
        //have it queued for emitting
        if (m_signal_field_changed_set.find(current) == m_signal_field_changed_set.end())
          {
            //it's not already marked, so add it to both data structures
            m_signal_field_changed_set.insert( current );
            m_signal_field_changed_queue.push( current );
          }
        return;
      }

    m_signal_field_changed_emitting = true;
    do
      {
        if (m_signal_field_changed_set.size() > 0)
          {
            current = m_signal_field_changed_queue.front();
            m_signal_field_changed_queue.pop();
            m_signal_field_changed_set.erase(current);
          }
          m_signal_field_changed.emit( FieldBuffer(*this, current) );
      }
    while (m_signal_field_changed_set.size() > 0);

    m_signal_field_changed_emitting = false;
    return;
  }

  void RecordBuffer::on_record_changed( )
  {
    m_signal_record_changed.emit();
  }

  sigc::signal<void> RecordBuffer::signal_record_changed( )
  {
    return m_signal_record_changed;
  }

  void RecordBuffer::set_data( void* data, size_t size, DataMode datamode )
  {
    Buffer::set_data(data, size, datamode);
    m_signal_record_changed.emit();
  }


}
